/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */

package escota.flickr;

import escota.utils.Log;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import javax.swing.ListModel;
import javax.swing.event.ListDataListener;

/**
 *
 * @author AGuS
 */
public class RuleList implements List<Rule>, ListModel {
    private List<Rule> ruleList;
    private final String file;

    
    public RuleList(String nsid)
    {
        ruleList = new LinkedList<Rule>();
        file = System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + nsid + "ruleList.bin";
    }

    public RuleList(List<Rule> ruleList, String nsid)
    {
        this.ruleList = ruleList;
        file = System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + nsid + "-ruleList.bin";
    }
    public void saveRuleListToDisk () throws IOException {
        ObjectOutputStream salida = new ObjectOutputStream(new FileOutputStream(file));
        salida.writeObject(ruleList);
        Log.debug("Rule list successfully saved to " + file);
    }


    public void loadRuleListFromDisk()
    {
        try
        {
            ObjectInputStream entrada = new ObjectInputStream(new FileInputStream(file));
            ruleList = (List<Rule>) entrada.readObject();
            Log.debug("Rule list successfully loaded from " + file);
        }
        catch(Exception e)
        {
            Log.debug("There no was a rule list saved");
 
        }
    }
    public boolean existRule(String name)
    {
        boolean exist = false;
        Iterator<Rule> it = this.iterator();
        while(it.hasNext())
        {
            if(it.next().getName().equalsIgnoreCase(name))
            {
                exist = true;
                break;
            }
        }
        return exist;
    }

    public int size() {
        return ruleList.size();
    }

    public boolean isEmpty() {
        return ruleList.isEmpty();
    }

    public boolean contains(Object arg0) {
        return ruleList.contains(arg0);
    }

    public Iterator<Rule> iterator() {
        return ruleList.iterator();
    }

    public Object[] toArray() {
        return ruleList.toArray();
    }

    public <T> T[] toArray(T[] arg0) {
        return ruleList.toArray(arg0);
    }

    public boolean add(Rule arg0) {
        return ruleList.add(arg0);
    }

    public boolean remove(Object arg0) {
        return ruleList.remove(arg0);
    }

    public boolean containsAll(Collection<?> arg0) {
        return ruleList.containsAll(arg0);
    }

    public boolean addAll(Collection<? extends Rule> arg0) {
        return ruleList.addAll(arg0);
    }

    public boolean addAll(int arg0, Collection<? extends Rule> arg1) {
        return ruleList.addAll(arg0, arg1);
    }

    public boolean removeAll(Collection<?> arg0) {
        return ruleList.removeAll(arg0);
    }

    public boolean retainAll(Collection<?> arg0) {
        return ruleList.retainAll(arg0);
    }

    public void clear() {
        ruleList.clear();
    }

    public Rule get(int arg0) {
        return ruleList.get(arg0);
    }

    public Rule set(int arg0, Rule arg1) {
        return ruleList.set(arg0, arg1);
    }

    public void add(int arg0, Rule arg1) {
        ruleList.add(arg0, arg1);
    }

    public Rule remove(int arg0) {
        return ruleList.remove(arg0);
    }

    public int indexOf(Object arg0) {
        return ruleList.indexOf(arg0);
    }

    public int lastIndexOf(Object arg0) {
        return ruleList.lastIndexOf(arg0);
    }

    public ListIterator<Rule> listIterator() {
        return ruleList.listIterator();
    }

    public ListIterator<Rule> listIterator(int arg0) {
        return ruleList.listIterator(arg0);
    }

    public List<Rule> subList(int arg0, int arg1) {
        return ruleList.subList(arg0, arg1);
    }

    public int getSize() {
        return this.size();
    }

    public Object getElementAt(int pos) {
        return this.get(pos);
    }

    public void addListDataListener(ListDataListener arg0) {
    }

    public void removeListDataListener(ListDataListener arg0) {
    }
    @Override
    public String toString()
    {
        String res = "";
        Iterator<Rule> it = this.iterator();
        while(it.hasNext())
        {
            res += it.next().getName();
            if(it.hasNext())
                res +=", ";
        }
        return res;
    }

}
