/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */

package escota.flickr;

import escota.utils.Log;
import com.aetrion.flickr.FlickrException;
import escota.EscotaTaskExecutor;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
import javax.swing.ListModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import org.xml.sax.SAXException;

/**
 *
 * @author AGuS
 */
public class TaskQueue implements Queue<Task>, ListModel, TableModel {

    private Queue<Task> taskQueue;
    private final String file;
    
    public void executeTasks(Conexion conexion, EscotaTaskExecutor te) throws IOException, SAXException, FlickrException
    {
        conexion.authorize(conexion.getNsid());
        String result = "";
        te.getText().setText(result);
        while(canDoAnyTask())
        {
            te.getProgressBar().setIndeterminate(true);
            Task task = taskQueue.poll();
            te.getLabel().setText("Executing: " + task.getPhotoTitle() + " --> " + task.getGroupName());
            if(task.canDoIt())
            {
                if(task.satisfiedRule(conexion))
                {
                    if(task.isIsAddition())
                    {
                        try
                        {
                            conexion.getFlickr().getPoolsInterface().add(task.getPhotoID(), task.getGroupID());
                            result += "Photo: " + task.getPhotoTitle() + " has been moved to group: " + task.getGroupName()  + "\n";
                            te.getText().setText(result);
                        }
                        catch(FlickrException e)
                        {
                           if(e.getErrorCode().equals("5"))
                            {
                                result += "Maximum amount for group " + task.getGroupName() + "has been reached, inserting the photo " + task.getPhotoTitle() + " to the queue again.\n";
                                te.getText().setText(result);
                                Log.debug("The user has already added the maximum amount of allowed photos to the group " + task.getGroupID());
                                task.addDays(1);
                                taskQueue.offer(task);
                            }
                            else if(e.getErrorCode().equals("3"))
                            {
                                Log.debug("The specified photo " + task.getPhotoID()  + " is already in the pool for the specified group " + task.getGroupID());
                                result += "The specified photo " + task.getPhotoTitle()  + " is already in the pool for the specified group " + task.getGroupName() + "\n";
                                te.getText().setText(result);
                            }
                            else
                            {
                               Log.debug("Error ejecutando la tarea " + task);
                                throw e;
                            }
                        }
                    }
                    else
                    {
                        //Delete
                        try
                        {
                            conexion.getFlickr().getPoolsInterface().remove(task.getPhotoID(), task.getGroupID());
                            result += "Photo: " + task.getPhotoTitle() + " has been removed from group: " + task.getGroupName()  + "\n";
                            te.getText().setText(result);
                        }
                        catch(FlickrException e)
                        {
                            //2: Photo not in pool
                            if(e.getErrorCode().equals("2"))
                            {
                                result += "The photo " + task.getPhotoTitle() + " was not in the pool of group " + task.getGroupName();
                                te.getText().setText(result);
                                Log.debug("The photo " + task.getPhotoTitle() + " was not in the pool of group " + task.getGroupName());
                            }
                            else
                            {
                               Log.debug("Error ejecutando la tarea " + task);
                                throw e;
                            }

                        }
                    }
                }
            }
            else
            {
                task.addDays(1);
                taskQueue.offer(task);
                Log.debug("Vuelta a la cola de tareas sumandole un dia a la Tarea " + task );
                result += "It is to soon for executing this task " + task.getPhotoTitle() + "\n";
                te.getText().setText(result);

            }
        }
        te.getProgressBar().setIndeterminate(false);
        te.getLabel().setText("Finished all executable tasks");

    }
     public boolean canDoAnyTask()
     {
         Iterator<Task> it = taskQueue.iterator();
         while(it.hasNext())
         {
             Task task = it.next();
             if(task.canDoIt())
                 return true;
         }
         return false;
     }

    public void saveTaskQueueToDisk () throws IOException {
        ObjectOutputStream salida = new ObjectOutputStream(new FileOutputStream(file));
        salida.writeObject(taskQueue);
        Log.debug("Task queue sucessifully saved to " + file);
    }
    public void loadTaskQueueFromDisk()
    {
        try
        {
            ObjectInputStream entrada = new ObjectInputStream(new FileInputStream(file));
            taskQueue = (Queue<Task>) entrada.readObject();
            Log.debug("Task queue sucessifully loaded from " + file);
        }
        catch(Exception e)
        {
            Log.debug("There no was a task queue saved");
        }
    }

    public TaskQueue(String nsid)
    {
        taskQueue = new LinkedList<Task>();
        file = System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + nsid + "taskQueue.bin";
    }
    public TaskQueue(Queue<Task> taskQueue, String nsid)
    {
        this.taskQueue = taskQueue;
        file = System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + nsid + "taskQueue.bin";
    }

    public boolean add(Task arg0) {
        Log.debug("Tarea añadida a la cola: " + arg0);
        return taskQueue.add(arg0);
    }

    public boolean offer(Task arg0) {
        return taskQueue.offer(arg0);
    }

    public Task remove() {
        return taskQueue.remove();
    }

    public Task poll() {
        return taskQueue.poll();
    }

    public Task element() {
        return taskQueue.element();
    }

    public Task peek() {
        return taskQueue.peek();
    }

    public int size() {
        return taskQueue.size();
    }

    public boolean isEmpty() {
        return taskQueue.isEmpty();
    }

    public boolean contains(Object arg0) {
        return taskQueue.contains(arg0);
    }

    public Iterator<Task> iterator() {
        return taskQueue.iterator();
    }

    public Object[] toArray() {
        return taskQueue.toArray();
    }

    public <T> T[] toArray(T[] arg0) {
        return taskQueue.toArray(arg0);
    }

    public boolean remove(Object arg0) {
        return taskQueue.remove(arg0);
    }

    public boolean containsAll(Collection<?> arg0) {
        return taskQueue.containsAll(arg0);
    }

    public boolean addAll(Collection<? extends Task> arg0) {
        return taskQueue.addAll(arg0);
    }

    public boolean removeAll(Collection<?> arg0) {
        return taskQueue.removeAll(arg0);
    }

    public boolean retainAll(Collection<?> arg0) {
        return taskQueue.retainAll(arg0);
    }

    public void clear() {
        taskQueue.clear();
    }
        public int getSize() {
        return this.size();
    }

    public Object getElementAt(int pos) {
        return this.getElementAt(pos);
    }

    public void addListDataListener(ListDataListener arg0) {
    }

    public void removeListDataListener(ListDataListener arg0) {
    }
    @Override
    public String toString()
    {
        String res = "Begin Task Queue ------------------------\n";
        Iterator it = this.iterator();
        while(it.hasNext())
        {
            Task task = (Task) it.next();
            res += task.getPhotoTitle() + "-->" + task.getGroupName() + "\n";
        }
        res += "End Task Queue --------------------------\n";
        return res;
    }

    public int getRowCount() {
        return size();
    }

    public int getColumnCount() {
        return 5;
    }

    public String getColumnName(int col) {
        switch ( col ) {
      case 0:
           return "Photo to move";
      case 1:
           return "Group to add photo";
      case 2:
           return "Rule to apply";
      case 3:
           return "Execute on";
      case 4:
           return "Operation";
      default:
           return "ERROR";
      }
    }

    public Class getColumnClass(int arg0) {
        return "".getClass();
    }

    public boolean isCellEditable(int arg0, int arg1) {
        return false;
    }

    public Object getValueAt(int row, int col) {
        Task temp = (Task) taskQueue.toArray()[row];
        switch (col) {
            case 0:
                return temp.getPhotoTitle();
            case 1:
                return temp.getGroupName();
            case 2:
                return temp.getRule().getName();
            case 3:
                DateFormat f = new SimpleDateFormat("dd/MM/yy");
                return f.format(temp.getNextTry());
            case 4:
                String operation = "";
                if(temp.isIsAddition())
                    operation = "Addition";
                else
                    operation = "Delete";
                return operation;
            default:
                return "ERROR";
        }
    }

    public void setValueAt(Object task, int row, int col) {
        offer((Task) task);
    }

    public void addTableModelListener(TableModelListener arg0) {
        Log.debug("method addTableModelListener not implemented yet");
    }

    public void removeTableModelListener(TableModelListener arg0) {
        Log.debug("method removeTableModelListener not implemented yet");
    }


}
